@@ -1,5 +1,5 @@
 /*
- *  $Id: ezdfile.c 7756 2007-03-25 08:10:20Z yeti-dn $
+ *  $Id: ezdfile.c 8736 2008-06-25 09:44:36Z xhorak $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -107,13 +107,15 @@ static GwyContainer* ezdfile_get_metadata  (GPtrArray *ezdfile,
 static void          fix_scales            (EZDSection *section,
                                             gint idx,
                                             GwyContainer *container);
+static void          check_section_ranges  (gpointer data, 
+                                            gpointer user_data);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Imports Nanosurf EZD and NID data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -194,6 +196,9 @@ ezdfile_load(const gchar *filename,
         return NULL;
     }
 
+    /* fix for ranges equal to zero */
+    g_ptr_array_foreach(ezdfile, &check_section_ranges, NULL);
+
     container = gwy_container_new();
     i = 0;
     for (n = 0; n < ezdfile->len; n++) {
@@ -365,16 +370,6 @@ file_read_header(GPtrArray *ezdfile,
         else
             g_hash_table_replace(section->meta, g_strdup(line), g_strdup(p));
     }
-
-    if (!((section->xrange.range = fabs(section->xrange.range)) > 0)) {
-        g_warning("Real x size is 0.0, fixing to 1.0");
-        section->xrange.range = 1.0;
-    }
-    if (!((section->yrange.range = fabs(section->yrange.range)) > 0)) {
-        g_warning("Real y size is 0.0, fixing to 1.0");
-        section->yrange.range = 1.0;
-    }
-
     return TRUE;
 }
 
@@ -606,5 +601,29 @@ read_data_field(GwyDataField *dfield,
         g_warning("Damn! Bit depth %d is not implemented", section->bitdepth);
 }
 
+static void
+check_section_ranges(gpointer data, gpointer user_data)
+{   
+   EZDSection *section = (EZDSection*) data;
+   
+   if (!section->data)
+      return;
+   
+   gwy_debug("Check ranges");
+   if (!((section->xrange.range = fabs(section->xrange.range)) > 0)) {
+       g_warning("Real x size is 0.0, fixing to 1.0");
+       //section->xrange.range = 1.0;
+   }
+   if (!((section->yrange.range = fabs(section->yrange.range)) > 0)) {
+       g_warning("Real y size is 0.0, fixing to 1.0");
+       //section->yrange.range = 1.0;
+   }
+   /* fix for Z axis*/
+   if (!((section->zrange.range = fabs(section->zrange.range)) > 0)) {
+       g_warning("Real z size is 0.0, fixing to 1.0");
+       section->zrange.range = 1.0;
+   }
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
 
