@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: rawfile.c 8528 2007-09-17 18:20:08Z yeti-dn $
+ *  @(#) $Id: rawfile.c 8717 2008-03-26 12:27:50Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -215,7 +215,7 @@ static GwyModuleInfo module_info = {
     N_("Imports raw data files, both ASCII and binary, according to "
        "user-specified format."),
     "Yeti <yeti@gwyddion.net>",
-    "2.3.1",
+    "2.4",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -1057,10 +1057,10 @@ rawfile_read_data_field(RawFileControls *controls,
     dfield = gwy_data_field_new(args->p.xres, args->p.yres,
                                 m*args->p.xreal, m*args->p.yreal,
                                 FALSE);
-    siunit = gwy_si_unit_new("m");
+    siunit = gwy_si_unit_new(args->p.xyunit);
     gwy_data_field_set_si_unit_xy(dfield, siunit);
     g_object_unref(siunit);
-    siunit = gwy_si_unit_duplicate(siunit);
+    siunit = gwy_si_unit_new(args->p.zunit);
     gwy_data_field_set_si_unit_z(dfield, siunit);
     g_object_unref(siunit);
 
