@@ -1,6 +1,5 @@
-#line 1 "pygwy.c"
 /*
- *  @(#) $Id: pygwy.c 8620 2007-10-10 12:17:33Z xhorak $
+ *  @(#) $Id: pygwy.c 8726 2008-04-28 14:23:32Z xhorak $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -42,12 +41,13 @@
 #include <libgwymodule/gwymodule-process.h>
 #include <libgwymodule/gwymodule-file.h>
 #include <app/gwyapp.h>
+#include "wrap_calls.h"
 
-static GValue*    convert_pyobject_to_gvalue  (PyObject *o);
-static PyObject*  convert_gvalue_to_pyobject  (GValue *value);
-
+static GValue*    convert_pyobject_to_gvalue         (PyObject *o);
+static PyObject*  convert_gvalue_to_pyobject         (GValue *value);
+static void       pygwy_create_py_list_of_containers (GwyContainer *data, gpointer list);
 #include "pygwywrap.c"
-#line 53 "pygwy.c"
+#line 52 "pygwy.c"
 
 typedef struct {
     gchar *name;
@@ -56,6 +56,13 @@ typedef struct {
     time_t m_time;
 } PygwyPluginInfo;
 
+typedef struct {
+   PyObject *std_err;
+   PyObject *dictionary;
+   GtkWidget *console_output;
+   GtkWidget *console_file_content;
+} PygwyConsoleSetup;
+
 typedef enum {
     PYGWY_PROCESS, PYGWY_FILE, PYGWY_GRAPH, PYGWY_LAYER, PYGWY_UNDEFINED
 } PygwyPluginType;
@@ -65,7 +72,13 @@ static gboolean         module_register       (void);
 static void             pygwy_proc_run        (GwyContainer *data,
                                                GwyRunType run,
                                                const gchar *name);
+static void             pygwy_graph_run       (GwyGraph *graph,
+                                               const gchar *name);
+static void             pygwy_console_run     (GwyContainer *data,
+                                               GwyRunType run,
+                                               const gchar *name);
 static void             pygwy_register_plugins(void);
+static void             pygwy_register_console(void);
 static PygwyPluginInfo* pygwy_find_plugin     (const gchar* name);
 static gboolean         pygwy_file_save_run   (GwyContainer *data,
                                                const gchar *filename,
@@ -80,10 +93,17 @@ static gint             pygwy_file_detect_run (const GwyFileDetectInfo
                                                *fileinfo,
                                                gboolean only_name,
                                                gchar *name);
+static void             pygwy_on_console_command_execute(GtkEntry *entry, 
+                                                         gpointer user_data);
 
+static gboolean         pygwy_on_console_close(GtkWidget *widget, 
+                                               GdkEvent *event, 
+                                               gpointer user_data);
 static GList *s_pygwy_plugins = NULL;
+static PygwyConsoleSetup *s_console_setup = NULL;
 static PyObject *s_pygwy_dict;
 static PyObject *s_main_module;
+const gchar pygwy_plugin_dir_name[] = "pygwy";
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -101,9 +121,11 @@ static gboolean
 module_register(void)
 {
     pygwy_register_plugins();
+    pygwy_register_console();
     return TRUE;
 }
 
+
 static void
 pygwy_initialize(void)
 {
@@ -133,7 +155,7 @@ pygwy_initialize(void)
 }
 
 static PyObject*
-pygwy_run_string(char *cmd, int type, PyObject *g, PyObject *l) {
+pygwy_run_string(const char *cmd, int type, PyObject *g, PyObject *l) {
     PyObject *ret = PyRun_String(cmd, type, g, l);
     if (!ret) {
         PyErr_Print();
@@ -183,7 +205,6 @@ pygwy_initialize_stderr_redirect(PyObject *d)
                      Py_file_input,
                      d,
                      d);
-    
 }
 
 static void
@@ -191,7 +212,6 @@ pygwy_finalize_stderr_redirect(PyObject *d)
 {
     PyObject *py_stderr;
     gchar *buf;
-    
     // rewind redirected stderr file, read its content and display it in error window
     pygwy_run_string("_stderr_redir.seek(0)\n"
                      "_stderr_str = _stderr_redir.read()\n"
@@ -209,7 +229,7 @@ pygwy_finalize_stderr_redirect(PyObject *d)
 }
 
 static PyObject *
-create_environment(const gchar *filename) {
+create_environment(const gchar *filename, gboolean show_errors) {
     PyObject *d, *plugin_filename;
     char *argv[1];
     argv[0] = NULL;
@@ -221,15 +241,17 @@ create_environment(const gchar *filename) {
     PySys_SetArgv(0, argv);
 
     // redirect stderr and stdout of python script to temporary file
-    pygwy_initialize_stderr_redirect(d);
+    if (show_errors)
+      pygwy_initialize_stderr_redirect(d);
     return d;
 }
 
 static void
-destroy_environment(PyObject *d) {
+destroy_environment(PyObject *d, gboolean show_errors) {
     // show content of temporary file which contains stderr and stdout of python
     // script and close it
-    pygwy_finalize_stderr_redirect(d);
+    if (show_errors)
+      pygwy_finalize_stderr_redirect(d);
     PyDict_Clear(d);
     Py_DECREF(d);
 }
@@ -277,7 +299,7 @@ pygwy_get_plugin_metadata(const gchar *filename,
         g_warning("Cannot read content of file '%s'", filename);
         return;
     }
-    d = create_environment(filename);
+    d = create_environment(filename, TRUE);
     if (!d) {
         g_warning("Cannot create copy of Python dictionary.");
         PyErr_Print();
@@ -318,6 +340,8 @@ pygwy_get_plugin_metadata(const gchar *filename,
         *type = PYGWY_FILE;
     } else if (g_ascii_strcasecmp ("GRAPH", type_str) == 0) {
         *type = PYGWY_GRAPH;
+    } else if (g_ascii_strcasecmp ("LAYER", type_str) == 0) {
+        *type = PYGWY_LAYER;
     } else {
         g_warning("Unknown type '%s' in '%s'", type_str, filename);
         *type = PYGWY_UNDEFINED;
@@ -329,7 +353,7 @@ error:
     }
     Py_XDECREF(plugin_module);
     g_free(plugin_file_content);
-    destroy_environment(d);
+    destroy_environment(d, TRUE);
 }
 
 static PygwyPluginInfo*
@@ -439,11 +463,56 @@ pygwy_register_proc_plugin(gchar *filename,
 
 }
 
+static gboolean
+pygwy_register_graph_plugin(gchar *filename,
+                            PyObject *code,
+                            gchar *name,
+                            gchar *menu_path)
+{
+    PygwyPluginInfo *info;
+
+    if (!code) {
+        g_warning("Cannot create code object for file '%s'", filename);
+        return FALSE;
+    }
+    if (!name) {
+        g_warning("Cannot register. Undefined 'plugin_name' variable in '%s'",
+                  filename);
+        return FALSE;
+    }
+    if (!menu_path) {
+        g_warning("Cannot register. Undefined 'plugin_desc' variable in '%s'",
+                  filename);
+        return FALSE;
+    }
+    info = pygwy_create_plugin_info(filename, name, code);
+
+    gwy_debug("Registering proc func.");
+    if (gwy_graph_func_register(info->name,
+                                pygwy_graph_run,
+                                menu_path,
+                                GWY_STOCK_GRAPH_FUNCTION,
+                                GWY_MENU_FLAG_GRAPH, // TODO: determine correct flag
+                                N_("Graph function written in Python")) ) { // not very descriptive
+        // append plugin to list of plugins
+        s_pygwy_plugins = g_list_append(s_pygwy_plugins, info);
+    } else {
+        // FIXME: Terminated by glib free(): invalid pointer: blabla
+        // when inserting duplicate module
+        // g_free(info->name);
+        // g_free(info->filename);
+        // g_free(info);
+        g_warning("Cannot register plugin '%s'", filename);
+        return FALSE;
+    }
+    return TRUE;
+
+}
+
 static void
 pygwy_register_plugins(void)
 {
     // FIXME: maybe place somewhere else
-    static const gchar pygwy_plugin_dir_name[] = "pygwy";
     GDir *plugin_dir;
     const gchar *plugin_filename;
     gchar *plugin_menu_path, *plugin_fullpath_filename;
@@ -535,6 +604,21 @@ pygwy_register_plugins(void)
                               "'plugin_type' variable  ('%s')",
                               plugin_fullpath_filename);
                     break;
+                case PYGWY_GRAPH:
+                    printf("%s %s\n", plugin_name, plugin_desc);
+                    if (plugin_code && plugin_name && plugin_desc) {
+                    pygwy_register_graph_plugin(plugin_fullpath_filename,
+                                                plugin_code,
+                                                plugin_name,
+                                                plugin_desc);
+
+                    } else {
+                        g_warning("Could not register graph module:"
+                                  " variables plugin_desc, plugin_type"
+                                  " and plugin_name not defined.");
+                    }
+
+                    break;
                 default:
                     g_warning("Rest of plugin types not yet implemented"); //TODO: PYGWY_GRAPH, PYGWY_LAYER
             }
@@ -630,7 +714,7 @@ pygwy_proc_run(GwyContainer *data, GwyRunType run, const gchar *name)
               info->filename);
 
     // create new environment
-    d = create_environment(info->filename);
+    d = create_environment(info->filename, TRUE);
     if (!d) {
         g_warning("Cannot create copy of Python dictionary.");
         return;
@@ -643,7 +727,7 @@ pygwy_proc_run(GwyContainer *data, GwyRunType run, const gchar *name)
     // import, execute the module and check for 'run' func
     module = PyImport_ExecCodeModule(info->name, info->code);
     if (!pygwy_check_func(module, "run", info->filename)) {
-        destroy_environment(d);
+        destroy_environment(d, TRUE);
         return;
     }
 
@@ -665,9 +749,65 @@ pygwy_proc_run(GwyContainer *data, GwyRunType run, const gchar *name)
 
     Py_DECREF(module);
     Py_DECREF(py_container); //FIXME
-    destroy_environment(d);
+    destroy_environment(d, TRUE);
 }
 
+static void
+pygwy_graph_run(GwyGraph *graph, const gchar *name)
+{
+    PygwyPluginInfo *info;
+    PyObject *py_graph, *module, *d;
+    gchar *cmd;
+
+    // find plugin
+    if (!(info = pygwy_find_plugin(name))) {
+        g_warning("Cannot find plugin '%s'.", name);
+        return;
+    }
+    gwy_debug("Running plugin '%s', filename '%s'",
+              info->name,
+              info->filename);
+
+    // create new environment
+    d = create_environment(info->filename, TRUE);
+    if (!d) {
+        g_warning("Cannot create copy of Python dictionary.");
+        return;
+    }
+
+    // check last and current file modification time and override
+    // the code if required
+    pygwy_reload_code(&info);
+    gwy_debug("Import module and check for 'run' func");
+    // import, execute the module and check for 'run' func
+    module = PyImport_ExecCodeModule(info->name, info->code);
+    if (!pygwy_check_func(module, "run", info->filename)) {
+        destroy_environment(d, TRUE);
+        return;
+    }
+
+    gwy_debug("Running plugin '%s', filename '%s'", info->name, info->filename);
+    // create graph named 'graph' to allow access the graph object from python
+    py_graph = pygobject_new((GObject*)graph);
+    if (!py_graph) {
+        g_warning("Variable 'gwy.data' was not inicialized.");
+    }
+    PyDict_SetItemString(s_pygwy_dict, "graph", py_graph);
+
+    // import module using precompiled code and run its 'run()' function
+    cmd = g_strdup_printf("import %s\n"
+                          "%s.run()",
+                          info->name,
+                          info->name);
+    pygwy_run_string(cmd, Py_file_input, d, d);
+    g_free(cmd);
+
+    Py_DECREF(module);
+    Py_DECREF(py_graph); //FIXME
+    destroy_environment(d, TRUE);
+}
+
+
 static gboolean
 pygwy_file_save_run(GwyContainer *data,
                     const gchar *filename,
@@ -690,7 +830,7 @@ pygwy_file_save_run(GwyContainer *data,
               info->name,
               info->filename);
     // create new environment
-    d = create_environment(info->filename);
+    d = create_environment(info->filename, TRUE);
     if (!d) {
         g_warning("Cannot create copy of Python dictionary.");
         return FALSE;
@@ -702,7 +842,7 @@ pygwy_file_save_run(GwyContainer *data,
     module = PyImport_ExecCodeModule(info->name, info->code);
     // check if load function is defined
     if (!pygwy_check_func(module, "save", info->filename)) {
-        destroy_environment(d);
+        destroy_environment(d, TRUE);
         return FALSE;
     }
     // create input container and put it into __main__ module dictionary
@@ -737,7 +877,7 @@ pygwy_file_save_run(GwyContainer *data,
     Py_XDECREF(module);
     Py_XDECREF(py_container); //FIXME
     Py_XDECREF(py_filename);
-    destroy_environment(d);
+    destroy_environment(d, TRUE);
     return res;
 }
 
@@ -763,7 +903,7 @@ pygwy_file_load_run(const gchar *filename,
               info->filename);
 
     // create new environment
-    d = create_environment(info->filename);
+    d = create_environment(info->filename, TRUE);
     if (!d) {
         g_warning("Cannot create copy of Python dictionary.");
         goto error;
@@ -811,7 +951,7 @@ error:
     Py_XDECREF(type);
     Py_XDECREF(module);
 
-    destroy_environment(d);
+    destroy_environment(d, TRUE);
     gwy_debug("Return value %p", res);
     return res;
 }
@@ -835,7 +975,7 @@ pygwy_file_detect_run(const GwyFileDetectInfo *fileinfo,
               info->name,
               info->filename);
     // create new environment
-    d = create_environment(info->filename);
+    d = create_environment(info->filename, TRUE);
     if (!d) {
         g_warning("Cannot create copy of Python dictionary.");
         return FALSE;
@@ -849,7 +989,7 @@ pygwy_file_detect_run(const GwyFileDetectInfo *fileinfo,
     // check if load function is defined
     if (!pygwy_check_func(module, "detect_by_name", info->filename)
        || !pygwy_check_func(module, "detect_by_content", info->filename)) {
-        destroy_environment(d);
+        destroy_environment(d, TRUE);
         return FALSE;
     }
 
@@ -896,7 +1036,7 @@ pygwy_file_detect_run(const GwyFileDetectInfo *fileinfo,
               res,
               info->name);
     Py_DECREF(module);
-    destroy_environment(d);
+    destroy_environment(d, TRUE);
     return res;
 
 }
@@ -1032,8 +1172,51 @@ convert_pyobject_to_gvalue(PyObject *o)
              || o->ob_type == &PyGwyDataField_Type
              || o->ob_type == &PyGwyDataLine_Type
              || o->ob_type == &PyGwyResource_Type
-             || o->ob_type == &PyGwySIUnit_Type
              || o->ob_type == &PyGwySelection_Type
+             || o->ob_type == &PyGwyAxis_Type
+             || o->ob_type == &PyGwyColorButton_Type
+             || o->ob_type == &PyGwyContainer_Type
+             || o->ob_type == &PyGwyCurve_Type
+             || o->ob_type == &PyGwyDataView_Type
+             || o->ob_type == &PyGwyDataViewLayer_Type
+             || o->ob_type == &PyGwyDataWindow_Type
+             || o->ob_type == &PyGwyGraph_Type
+             || o->ob_type == &PyGwyGraphArea_Type
+             || o->ob_type == &PyGwyGraphCorner_Type
+             || o->ob_type == &PyGwyGraphCurveModel_Type
+             || o->ob_type == &PyGwyGraphCurves_Type
+             || o->ob_type == &PyGwyGraphData_Type
+             || o->ob_type == &PyGwyGraphLabel_Type
+             || o->ob_type == &PyGwyGraphModel_Type
+             || o->ob_type == &PyGwyGraphWindow_Type
+             || o->ob_type == &PyGwyInventory_Type
+             || o->ob_type == &PyGwyMarkerBox_Type
+             || o->ob_type == &PyGwyNullStore_Type
+             || o->ob_type == &PyGwyPixmapLayer_Type
+             || o->ob_type == &PyGwyLayerBasic_Type
+             || o->ob_type == &PyGwyLayerMask_Type
+             || o->ob_type == &PyGwyHMarkerBox_Type
+             || o->ob_type == &PyGwyResource_Type
+             || o->ob_type == &PyGwyNLFitPreset_Type
+             || o->ob_type == &PyGwyFDCurvePreset_Type
+             || o->ob_type == &PyGwyGLMaterial_Type
+             || o->ob_type == &PyGwyCDLine_Type
+             || o->ob_type == &PyGwyRuler_Type
+             || o->ob_type == &PyGwyHRuler_Type
+             || o->ob_type == &PyGwySIUnit_Type
+             || o->ob_type == &PyGwySciText_Type
+             || o->ob_type == &PyGwySpectra_Type
+             || o->ob_type == &PyGwySelectionGraph1DArea_Type
+             || o->ob_type == &PyGwySelectionGraphArea_Type
+             || o->ob_type == &PyGwySelectionGraphLine_Type
+             || o->ob_type == &PyGwySelectionGraphPoint_Type
+             || o->ob_type == &PyGwySelectionGraphZoom_Type
+             || o->ob_type == &PyGwySensitivityGroup_Type
+             || o->ob_type == &PyGwyShader_Type
+             || o->ob_type == &PyGwyStatusbar_Type
+             || o->ob_type == &PyGwyStringList_Type
+             || o->ob_type == &PyGwyVRuler_Type
+             || o->ob_type == &PyGwyVectorLayer_Type
              || o->ob_type == &PyGwySpectra_Type) {
         GObject *d;
 
@@ -1048,4 +1231,310 @@ convert_pyobject_to_gvalue(PyObject *o)
     return g_value;
 }
 
+static void
+pygwy_register_console()
+{
+
+    if (gwy_process_func_register(N_("Pygwy console"),
+                                  pygwy_console_run,
+                                  N_("/Pygwy console"),
+                                  NULL,
+                                  GWY_RUN_IMMEDIATE,
+                                  GWY_MENU_FLAG_DATA,
+                                  N_("Python wrapper console")) ) {
+
+    }
+}
+
+char *
+pygwy_console_run_command(gchar *cmd, int mode)
+{
+   if (!cmd) {
+      g_warning("No command.");
+      return NULL;
+   }
+
+   if (!s_console_setup) {
+      g_warning("Console setup structure is not defined!");
+      return NULL;
+   }
+   // store _stderr_redir location
+   pygwy_run_string(cmd, 
+         mode, 
+         s_console_setup->dictionary, 
+         s_console_setup->dictionary);
+   pygwy_run_string("_stderr_redir_pos = _stderr_redir.tell()\n"
+                    "_stderr_redir.seek(0)\n"
+                    "_stderr_redir_string = _stderr_redir.read(_stderr_redir_pos)\n"
+                    "_stderr_redir.seek(0)",
+         Py_file_input, 
+         s_console_setup->dictionary,
+         s_console_setup->dictionary);
+
+   return PyString_AsString( PyDict_GetItemString(
+            s_console_setup->dictionary, 
+            "_stderr_redir_string") );
+}
+
+static void
+pygwy_console_append(gchar *msg)
+{
+   GtkTextBuffer *console_buf;
+   GtkTextIter start_iter, end_iter;
+   GString *output;
+   GtkTextMark *end_mark;
+
+   if (!msg) {
+      g_warning("No message to append.");
+      return;
+   }
+   if (!s_console_setup) {
+      g_warning("Console setup structure is not defined!");
+      return;
+   }
+   // read string which contain last command output
+   console_buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(s_console_setup->console_output));
+   gtk_text_buffer_get_bounds(console_buf, &start_iter, &end_iter);
+
+   // get output widget content
+   output = g_string_new (gtk_text_buffer_get_text(console_buf, &start_iter, &end_iter, FALSE));
+
+   // append input line
+   output = g_string_append(output, msg);
+   gtk_text_buffer_set_text (GTK_TEXT_BUFFER (console_buf), output->str, -1);
+   g_string_free(output, TRUE);
+
+   // scroll to end
+   gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(console_buf), &end_iter);
+   end_mark = gtk_text_buffer_create_mark(console_buf, "cursor", &end_iter, FALSE);
+   g_object_ref(end_mark);
+   gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(s_console_setup->console_output), 
+         end_mark, 0.0, FALSE, 0.0, 0.0);
+   g_object_unref(end_mark);
+
+}
+
+static void
+pygwy_on_console_run_file(GtkToolButton *btn, gpointer user_data)
+{
+   GtkTextIter start_iter, end_iter;
+   char *output, *file_info_line;
+
+   GtkTextBuffer *console_file_buf = 
+      gtk_text_view_get_buffer(GTK_TEXT_VIEW(s_console_setup->console_file_content));
+
+   file_info_line = g_strdup_printf(">>> Running file content of below textfield\n");
+
+   pygwy_console_append(file_info_line);
+
+   gtk_text_buffer_get_bounds(console_file_buf, &start_iter, &end_iter);
+   output = pygwy_console_run_command(
+         gtk_text_buffer_get_text(console_file_buf, &start_iter, &end_iter, FALSE), 
+         Py_file_input);
+   pygwy_console_append(output);
+
+   // get output widget content
+   gtk_text_buffer_get_text(console_file_buf, &start_iter, &end_iter, FALSE);
+
+}
+
+
+static void
+pygwy_on_console_open_file(GtkToolButton *btn, gpointer user_data)
+{
+   GtkFileChooserDialog *file_chooser;
+   GtkFileFilter *filter = gtk_file_filter_new();
+   GtkTextBuffer *console_file_buf;
+   gtk_file_filter_add_mime_type(filter, "text/x-python");
+
+   file_chooser = gtk_file_chooser_dialog_new("Run Python script", 
+         NULL, 
+         GTK_FILE_CHOOSER_ACTION_OPEN,
+         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+         NULL);
+   gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(file_chooser), filter);
+   if (gtk_dialog_run (GTK_DIALOG (file_chooser)) == GTK_RESPONSE_ACCEPT)
+   {
+      char *filename, *output, *file_content;
+      GError *err = NULL;
+      
+
+      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+      printf("Open: %s\n", filename);
+      if (!g_file_get_contents(filename,
+                            &file_content,
+                            NULL,
+                            &err)) {
+         g_warning("Cannot read content of file '%s'", filename);
+         return;
+      }
+      
+      // read string which contain last command output
+      console_file_buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(s_console_setup->console_file_content));
+
+      // append input line
+      gtk_text_buffer_set_text (GTK_TEXT_BUFFER (console_file_buf), file_content, -1);
+
+      g_free(file_content);
+      g_free(filename);
+   }
+   gtk_widget_destroy (file_chooser);
+}
+   
+static void 
+pygwy_console_create_gui()
+{
+   GtkWidget *console_win, *vbox1, *console_scrolledwin, *file_scrolledwin;
+   GtkWidget *entry_input, *button_bar, *button_open, *button_run;
+
+   // create static structure;
+   s_console_setup = g_malloc(sizeof(PygwyConsoleSetup));
+   // create GUI
+   console_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+   gtk_window_set_title (GTK_WINDOW (console_win), "Pygwy Console");
+
+   vbox1 = gtk_vbox_new (FALSE, 0);
+   gtk_container_add (GTK_CONTAINER (console_win), vbox1);
+
+   // buttons
+   button_open = gtk_tool_button_new_from_stock(GTK_STOCK_OPEN);
+   button_run = gtk_tool_button_new_from_stock(GTK_STOCK_EXECUTE);
+   button_bar = gtk_toolbar_new();
+   gtk_toolbar_insert(GTK_TOOLBAR(button_bar), button_open, 0);
+   gtk_toolbar_insert(GTK_TOOLBAR(button_bar), button_run, 0);
+   gtk_box_pack_start(GTK_BOX(vbox1), button_bar, FALSE, FALSE, 0);
+
+   // window
+   file_scrolledwin = gtk_scrolled_window_new (NULL, NULL);
+   gtk_box_pack_start (GTK_BOX (vbox1), file_scrolledwin, TRUE, TRUE, 0);
+   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (file_scrolledwin), GTK_SHADOW_IN);
+
+   console_scrolledwin = gtk_scrolled_window_new (NULL, NULL);
+   gtk_box_pack_start (GTK_BOX (vbox1), console_scrolledwin, TRUE, TRUE, 0);
+   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (console_scrolledwin), GTK_SHADOW_IN);
+
+
+   // console output
+   s_console_setup->console_output = gtk_text_view_new ();
+   gtk_container_add (GTK_CONTAINER (console_scrolledwin), s_console_setup->console_output);
+   gtk_text_view_set_editable (GTK_TEXT_VIEW (s_console_setup->console_output), FALSE);
+
+   // file buffer
+   s_console_setup->console_file_content = gtk_text_view_new();
+   gtk_container_add (GTK_CONTAINER (file_scrolledwin), s_console_setup->console_file_content);
+   gtk_text_view_set_editable (GTK_TEXT_VIEW (s_console_setup->console_file_content), TRUE);
+
+   entry_input = gtk_entry_new ();
+   gtk_box_pack_start (GTK_BOX (vbox1), entry_input, FALSE, FALSE, 0);
+   gtk_entry_set_invisible_char (GTK_ENTRY (entry_input), 9679);
+   gtk_widget_grab_focus(GTK_WIDGET(entry_input));
+
+   // entry widget on ENTER
+   g_signal_connect ((gpointer) entry_input, "activate",
+         G_CALLBACK (pygwy_on_console_command_execute),
+         NULL);
+   // open script signal connect
+   g_signal_connect ((gpointer) button_open, "clicked",
+         G_CALLBACK (pygwy_on_console_open_file),
+         NULL);
+   g_signal_connect ((gpointer) button_run, "clicked",
+         G_CALLBACK (pygwy_on_console_run_file),
+         NULL);
+
+   // connect on window close()
+   g_signal_connect ((gpointer) console_win, "delete_event",
+         G_CALLBACK (pygwy_on_console_close),
+         NULL);
+   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(s_console_setup->console_output), GTK_WRAP_WORD_CHAR);
+   gtk_window_resize(GTK_WINDOW(console_win), 600, 500);
+   gtk_widget_show_all(console_win);
+}
+
+static void
+pygwy_console_run(GwyContainer *data, GwyRunType run, const gchar *name)
+{
+    PyObject *d, *py_container;
+    gchar *plugin_dir_name = NULL, *sys_path_append;
+    
+    pygwy_initialize();      
+    pygwy_console_create_gui();
+    // create new environment    
+    d = create_environment("__console__", FALSE);
+    if (!d) {
+        g_warning("Cannot create copy of Python dictionary.");
+        return;
+    }
+
+    // create container named 'data' to allow access the container from python
+    py_container = pygobject_new((GObject*)data);
+    if (!py_container) {
+        g_warning("Variable 'gwy.data' was not inicialized.");
+    }
+    PyDict_SetItemString(s_pygwy_dict, "data", py_container);
+    
+    // redirect stdout & stderr to temporary file
+    pygwy_run_string("import sys, gwy, tempfile\n"
+                     "_stderr_redir = tempfile.TemporaryFile()\n"
+                     "sys.stderr = _stderr_redir\n"
+                     "sys.stdout = _stderr_redir\n",
+                     Py_file_input,
+                     d,
+                     d);
+
+    // add .gwyddion/pygwy to sys.path
+    plugin_dir_name = g_build_filename(gwy_get_user_dir(),
+                                       pygwy_plugin_dir_name,
+                                       NULL);
+    sys_path_append = g_strdup_printf("sys.path.append('%s')\n", plugin_dir_name);
+    pygwy_run_string(sys_path_append, Py_file_input, d, d);
+    g_free(sys_path_append);
+    g_free(plugin_dir_name);
+
+    // store values for closing console
+    s_console_setup->std_err = PyDict_GetItemString(d, "_stderr_redir");
+    Py_INCREF(s_console_setup->std_err);
+    s_console_setup->dictionary = d;
+}
+
+static void
+pygwy_on_console_command_execute(GtkEntry *entry, gpointer user_data)
+{
+    gchar *input_line;
+    GString *output;
+
+    input_line = g_strconcat(">>> ", gtk_entry_get_text(entry), "\n", NULL);
+    output = g_string_new(input_line);
+    output = g_string_append(output,  
+          pygwy_console_run_command(gtk_entry_get_text(entry), Py_single_input) );
+
+    pygwy_console_append(output->str);
+    g_string_free(output, TRUE);
+
+    gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
+}
+
+static gboolean
+pygwy_on_console_close(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+    //Py_DECREF(module);
+    //Py_DECREF(py_container); //FIXME
+    Py_DECREF(s_console_setup->std_err);
+    destroy_environment(s_console_setup->dictionary, FALSE);
+    g_free(s_console_setup);
+    return FALSE;
+}
+
+// function to fill list of containers, for  gwy_app_data_browser_get_containers
+static void
+pygwy_create_py_list_of_containers(GwyContainer *data, gpointer list)
+{
+
+   if (PyList_Append((PyObject *) list, pygobject_new((GObject *)data)) < 0) {
+      g_warning("Could not append container to python list of containers.");
+   }
+   
+}
+
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
